package geniusweb.opponentmodel.bayesian;

/**
 * A hypothesis is a value with a probability attached to it.
 * <p>
 * DO NOT USE THIS. Private use for {@link BayesianOpponentModel}.
 */

abstract class Hypothesis {
	final private double p;

	/**
	 * 
	 * @param p the probability of the item
	 */
	public Hypothesis(double p) {
		if (p < 0 || p > 1)
			throw new IllegalArgumentException("p must be in [0,1], got " + p);
		this.p = p;
	}

	/**
	 * 
	 * @return the probability that this hypothesis holds
	 */
	public double getProbability() {
		return p;
	}

	/**
	 * Subclasses implement this. We can not implement it here because
	 * subclasses return a more specific class than Hypothesis.
	 * 
	 * @param p the new probability value.
	 * @return new Hypothesis with p set to this value (and the rest unchanged).
	 */
	public abstract Hypothesis with(double p);

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		long temp;
		temp = Double.doubleToLongBits(p);
		result = prime * result + (int) (temp ^ (temp >>> 32));
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Hypothesis other = (Hypothesis) obj;
		if (Double.doubleToLongBits(p) != Double.doubleToLongBits(other.p))
			return false;
		return true;
	}

}
